<?php
if ( ! class_exists( 'Create_Tax_Lookup_Table' ) ) :

    class Create_Tax_Lookup_Table {

        private $migration_version = '0.4';

        function __construct() {

        }

        function run_migration() {
            $wclsi_db_version = get_option( WCLSI_DB_VERSION_OPTION, '0.0' );
            if ( version_compare( $wclsi_db_version, $this->migration_version ) < 0 ) {
                $this->create_tax_lookup_table();
                update_option( WCLSI_DB_VERSION_OPTION, $this->migration_version );
            }
        }

        private function table_exists( $table_name ) {
            global $wpdb;
            $sql = $wpdb->prepare( "SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s;", DB_NAME, $table_name );
            $row = $wpdb->get_row( $sql );

            return ! is_null( $row );
        }

        function create_tax_lookup_table() {
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

            global $wpdb;

            $table_name = $wpdb->prefix . "wclsi_tax_lookup";


            if ( $this->table_exists( $table_name ) ) {
                return;
            }

            $charset_collate = $wpdb->get_charset_collate();

            $sql = "
                    CREATE TABLE $table_name (
                        id                          BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
                        lightspeed_category_name    VARCHAR(100),
                        woocommerce_tax_rate_id     VARCHAR(100) UNIQUE NOT NULL,
                        lightspeed_tax_category_id  BIGINT(20) UNSIGNED UNIQUE NOT NULL,
                        updated_at                  TIMESTAMP NOT NULL DEFAULT NOW() ON UPDATE NOW(),
                        created_at                  TIMESTAMP NOT NULL DEFAULT NOW(),
                        PRIMARY KEY(id)
				    ) $charset_collate;			
				";

            dbDelta( $sql );

            $this->add_indexes(
                $table_name,
                array(
                    'woocommerce_tax_rate_id',
                    'lightspeed_tax_category_id'
                )
            );
        }

        private function add_indexes( $table_name, $indexes ) {
            global $wpdb;
            foreach ( $indexes as $index_name ) {
                $wpdb->query( "ALTER TABLE $table_name ADD INDEX ($index_name);" );
            }
        }
    }


endif;