<?php
if ( ! class_exists( 'WCLSI_Extend_WC_Products_Table' ) ) {
    /**
     * Class WCLSI_Extend_WC_Products_Table
     */
    class WCLSI_Extend_WC_Products_Table {
        public function __construct() {
            add_filter( 'bulk_actions-edit-product', array( $this, 'extend_bulk_actions' ), 10, 1 );
            add_filter( 'handle_bulk_actions-edit-product', array( $this, 'extend_handle_bulk_actions' ), 10, 3 );
            add_filter( 'manage_product_posts_columns', array( $this, 'extend_columns' ), 10, 1 );
            add_filter( 'manage_edit-product_sortable_columns', array( $this, 'extend_define_sortable_columns' ) );
            add_action( 'manage_product_posts_custom_column', array( $this, 'render_wclsi_columns' ), 10, 2 );
            add_action( 'pre_get_posts', array( $this, 'sort_wclsi_linked_prods' ) );
        }

        function sort_wclsi_linked_prods( $query ) {
            if ( ! is_admin() ) {
                return;
            }

            $orderby = $query->get( 'orderby' );
            $order   = $query->get( 'order' );
            if ( 'wclsi_linked' == $orderby ) {
                $query->set( 'meta_key', WCLSI_SYNC_POST_META );
                $title_order = $order === 'ASC' ? 'DESC' : 'ASC';
                $query->set( 'orderby', array( 'meta_value' => $order, 'title' => $title_order ) );
            }
        }

        function extend_define_sortable_columns( $columns ) {
            $custom = array(
                'wclsi_linked' => 'wclsi_linked'
            );

            return wp_parse_args( $custom, $columns );
        }

        function extend_columns( $columns ) {
            $columns['wclsi_linked'] = "<span class='th_wclsi_linked tips' data-tip='Linked to Lightspeed'>Linked to Lightspeed</span>";

            return $columns;
        }

        function render_wclsi_columns( $column, $wc_prod_id ) {
            if ( 'wclsi_linked' === $column ) {
                if ( WCLSI_Lightspeed_Prod::is_imported( $wc_prod_id ) && get_post_meta( $wc_prod_id, WCLSI_SYNC_POST_META, true ) ) {
                    echo "<span class='help_tips tips' style='font-size: 25px' data-tip='This product is linked with Lightspeed'><span class=\"dashicons dashicons-update-alt\"></span></span>";
                }
            }
        }

        function extend_bulk_actions( $actions ) {
            $actions['wclsi_link_to_ls'] = 'Link to Lightspeed';

            return $actions;
        }

        function extend_handle_bulk_actions( $redirect_to, $action, $ids ) {
            if ( 'wclsi_link_to_ls' === $action ) {
                if ( ! empty( $ids ) ) {
                    foreach ( $ids as $wc_prod_id ) {
                        as_schedule_single_action(
                            time() + (int) ( wclsi_get_api_wait_time_ms() / 1000 ),
                            'wclsi_link_wc_prod_to_lightspeed',
                            array( 'wc_prod_id' => $wc_prod_id, 'retry_count' => 0 ),
                            'wclsi'
                        );
                    }
                }
            }

            return $redirect_to;
        }
    }

    $WCLSI_Extend_WC_Products_Table = new WCLSI_Extend_WC_Products_Table();
}
