<?php
/**
 * @class LSI_Import_page
 * Displays to the user products and other settings that can be imported from LightSpeed as well as handles
 * a lot of the logic for importing products and other data from LightSpeed.
 *
 * @url http://cloud-docs.merchantos.com/API/APIHelp.help
 *
 *
 */
if ( ! class_exists( 'LSI_Import_Page' ) ) :

    class LSI_Import_Page {

        public $wclsi_import_table;
        public $ls_settings;
        public $table_disabled;

        function __construct() {
            global $WCLSI;

            add_action( 'admin_menu', array( $this, 'add_lightspeed_menu' ) );
            add_action( 'admin_menu', array( $this, 'add_import_page_submenu_item' ) );
            add_action( 'admin_menu', array( $this, 'add_settings_page_submenu_item' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_resources' ) );
            add_action( 'add_meta_boxes_product', array( $this, 'add_wclsi_meta_box' ), 10 );

            $this->ls_settings    = $WCLSI;
            $this->table_disabled =
                ! wclsi_oauth_enabled() ||
                ( false === $this->ls_settings->ls_account_id ) ||
                ! isset( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) ||
                ! ( isset( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) &&
                    is_numeric( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) &&
                    ( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] >= 0 ) );
        }

        function add_lightspeed_menu() {
            add_menu_page(
                'Lightspeed',
                'Lightspeed',
                'manage_woocommerce',
                'wclsi',
                array( $this, 'lsi_import_page_html' ),
                plugins_url( 'assets/img/ls-icon20x20.png', dirname( __FILE__ ) ),
                56.5
            );
        }

        /**
         * Create a submenu page for the LightSpeed Importer
         */
        public function add_import_page_submenu_item() {
            $hook = add_submenu_page(
                'wclsi',
                WCLSI_PROD_IMPORTER_MENU_NAME,
                WCLSI_PROD_IMPORTER_MENU_NAME,
                'manage_woocommerce',
                'wclsi',
                array( $this, 'lsi_import_page_html' ),
                1
            );

            add_action( "load-$hook", array( $this, 'add_screen_options' ) );
            add_action( "load-$hook", array( $this, 'prepare_lightspeed_import_table' ) );
        }

        public function add_settings_page_submenu_item() {
            add_submenu_page(
                'wclsi',
                WCLSI_SETTINGS_MENU_NAME,
                WCLSI_SETTINGS_MENU_NAME,
                'manage_woocommerce',
                WCLSI_ADMIN_SETTINGS_URL
            );
        }

        /**
         * Prepare the Lightspeed import table, make sure that runs after add_screen_options so screen options
         * will work correctly!
         */
        function prepare_lightspeed_import_table() {
            $this->wclsi_import_table = new WC_LS_Import_Table();

            if ( ! $this->table_disabled ) {
                $this->wclsi_import_table->prepare_items();
            }

            $this->check_setting_errors();
        }

        /**
         * Adds a screen options
         */
        function add_screen_options() {
            $option = 'per_page';
            $args   = array(
                'label'   => 'Lightspeed Products',
                'default' => 20,
                'option'  => 'product_per_page'
            );
            add_screen_option( $option, $args );
        }

        /**
         * Adds meta boxes to products
         *
         * @param $post
         */
        public function add_wclsi_meta_box( $post ) {
            $allowed_statuses = array( 'publish', 'future', 'draft', 'pending', 'private' );
            if ( isset( $post->post_status ) && in_array( $post->post_status, $allowed_statuses ) ) {
                add_meta_box(
                    'wclsi_meta_box',
                    __( 'Lightspeed Settings', 'woocommerce-lightspeed-pos' ),
                    array( $this, 'render_meta_box' ),
                    $post->post_type,
                    'side'
                );
            }
        }

        /**
         * Renders lightspeed settings in meta box
         *
         * @param $post
         */
        public function render_meta_box( $post ) {
            $wc_prod = wc_get_product( $post->ID );
            $ls_obj  = new WCLSI_Lightspeed_Prod();
            $ls_obj->init_via_wc_prod_id( $wc_prod->get_id() );
            if ( 0 === $ls_obj->id ) {
                if ( $wc_prod->is_type( 'simple' ) ) {
                    $ls_item_id = $wc_prod->get_meta( WCLSI_SINGLE_ITEM_ID_POST_META );
                    if ( $ls_item_id > 0 ) {
                        $ls_obj->init_via_item_id( $ls_item_id );
                    }
                } elseif ( $wc_prod->is_type( 'variable' ) ) {
                    $ls_item_matrix_id = $wc_prod->get_meta( WCLSI_MATRIX_ID_POST_META );
                    if ( $ls_item_matrix_id > 0 ) {
                        $ls_obj->init_via_item_matrix_id( $ls_item_matrix_id );
                    }
                }
            }

            if ( $ls_obj->id > 0 ) {
                $this->render_synced_meta_box( $post, $ls_obj );
            } else {
                $this->render_non_synced_meta_box( $post );
            }

            wp_nonce_field( 'wclsi_ajax', 'wclsi_nonce', false );
        }

        private function render_non_synced_meta_box( $post ) {
            $tooltip_img = esc_attr( WC()->plugin_url() . '/assets/images/help.png' );
            $tooltip     = __(
                'Create this product in Lightspeed. This does not check whether the product already exists in Lightspeed. ' .
                'If the product already exists, then this will result in a duplicate.',
                'woocommerce-lightspeed-pos'
            );

            echo "<p style='display: inline-block;'>" .
                 "<button class='button-secondary button' type='button' data-prodid='{$post->ID}' id='wclsi-sync-to-ls' style='float: left;'>Create this product in Lightspeed</button>" .
                 "<img class='help_tip tips wclsi-load-prod-tip' alt='{$tooltip}' data-tip='{$tooltip}' src='{$tooltip_img}' height='16' width='16'>" .
                 "</p>" .
                 "<div id='wclsi-sync-status'></div>";

            $this->render_relink_button( $post );
        }

        private function render_synced_meta_box( $post, $ls_obj ) {
            $is_synced = get_post_meta( $post->ID, WCLSI_SYNC_POST_META, true );
            $checked   = $is_synced ? 'checked' : '';

            echo '<p><input id="wclsi_ls_sync" class="wclsi-sync-cb" type="checkbox" data-prodid="' . esc_attr( $post->ID ) . '" ' . $checked . '><label for="wclsi_ls_sync">' . __( 'Link with Lightspeed?', 'woocommerce-lightspeed-pos' ) . '</label></p>';

            if ( $is_synced ) {
                $last_sync_date = ! empty( $ls_obj->wclsi_last_sync_date ) ? date_i18n( 'Y-m-d H:i', strtotime( $ls_obj->wclsi_last_sync_date ) ) : date_i18n( 'Y-m-d H:i', strtotime( $ls_obj->wclsi_last_import_date ) );
                echo sprintf( '<p>%s<b>%s</b></p>', __( 'Last Sync Date: ', 'woocommerce-lightspeed-pos' ), $last_sync_date );
            }

            echo '<button class="button-secondary button" type="button" data-prodid="' . esc_attr( $post->ID ) . '" id="wclsi-manual-sync">Manual Update via Lightspeed</button>';
            $this->render_relink_button( $post );
        }

        private function render_relink_button( $post ) {
            $tooltip_msg = __(
                'Link this product with an existing Lightspeed product. ' .
                'It will attempt to match the WooCommerce SKU with the Lightspeed System/Custom SKU in Lightspeed. ' .
                'Please make sure SKU values are unique to each product!',
                'woocommerce-lightspeedpos'
            );

            $img_src = esc_url( WC()->plugin_url() . '/assets/images/help.png' );
            $tooltip =
                "<span class='tips' data-tip='{$tooltip_msg}'>" .
                "<img alt='{$tooltip_msg}' class='help_tip' src='{$img_src}' height='16' width='16'>" .
                '</span>';

            $wc_prod = wc_get_product( $post->ID );
            if ( $wc_prod ) {
                if ( ! WCLSI_Lightspeed_Prod::is_imported( $wc_prod->get_id() ) ) {
                    $link_note =
                        $wc_prod->is_type( 'variable' ) ?
                            __( 'Note: to link Variable Products with Lightspeed, please input the Lightspeed Matrix ID under Inventory &#8594; SKU.', 'woocommerce-lightspeed-pos' ) : '';

                    echo sprintf(
                        '<div id="wclsi-relink-wrapper"><p><a id="wclsi-relink" data-prod-id="%d" href="#">%s</a>%s</p><p>%s</p></div>',
                        $post->ID,
                        __( 'Link with Lightspeed', 'woocommerce-lightspeed-pos' ),
                        $tooltip,
                        $link_note
                    );
                }
            }
        }

        /**
         * Enqueue necessary css and js files
         */
        public function enqueue_resources() {
            $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
            wp_enqueue_style( 'import-admin-page-css', plugins_url( "/assets/css/import-admin-page{$suffix}.css", dirname( __FILE__ ) ), array( 'thickbox' ), WCLSI_VERSION );
            wp_enqueue_style( 'woocommerce_admin_styles', WC()->plugin_url() . '/assets/css/admin.css', array(), WC()->version );
            wp_enqueue_script( 'jquery-tiptip', WC()->plugin_url() . "/assets/js/jquery-tiptip/jquery.tipTip{$suffix}.js", array( 'jquery' ), WC()->version, true );
        }

        /**
         * Makes sure that the user has initialized API vars...
         */
        public function check_setting_errors() {
            // Will only display errors if the plugin has initialized
            $wclsi_initialized = get_option( 'wclsi_initialized' );
            if ( ! $wclsi_initialized ) {
                add_settings_error(
                    'wclsi_settings',
                    'null_api_key',
                    sprintf( __( 'Thank you for using LightSpeed POS for WooCommerce! Please visit the <a href="%s">integration page</a> and authorize with Lightspeed to start the syncing process :).', 'woocommerce-lightspeed-pos' ), admin_url( 'admin.php?page=wc-settings&tab=integration&section=lightspeed-integration' ) ),
                    'updated'
                );

                return;
            }

            // Will only display one error at a time
            if ( ! wclsi_oauth_enabled() ) {
                $this->wclsi_import_table->items = array(); // Force the table to not display
                add_settings_error(
                    'wclsi_settings',
                    'wclsi_ouath_not_enabled',
                    sprintf( __( 'Please enabled your connection to Lightspeed! Please visit the <a href="%s">integration page</a> and connect to Lightspeed to start the syncing process.', 'woocommerce-lightspeed-pos' ), WCLSI_ADMIN_SETTINGS_URL )
                );
            } elseif ( empty( $this->ls_settings->store_timezone ) ) {
                $this->wclsi_import_table->items = array(); // Force the table to not display
                add_settings_error(
                    'wclsi_settings',
                    'null_timezone',
                    sprintf( __( 'Could not find store timezone. This is required for syncing products. Please visit the <a href="%s">integration page</a> and initialize the API settings.', 'woocommerce-lightspeed-pos' ), WCLSI_ADMIN_SETTINGS_URL )
                );
            } elseif ( ! isset( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) ||
                       ! ( isset( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) &&
                           is_numeric( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] ) &&
                           ( $this->ls_settings->settings[ WCLSI_INVENTORY_SHOP_ID ] >= 0 ) ) ) {

                $this->wclsi_import_table->items = array(); // Force the table to not display
                add_settings_error(
                    'wclsi_settings',
                    'null_inventory_shop_id',
                    sprintf( __( 'Could not find a primary lightspeed inventory shop id. Please visit the <a href="%s">integration page</a> and select a Primary Lightspeed Inventory Store.', 'woocommerce-lightspeed-pos' ), WCLSI_ADMIN_SETTINGS_URL )
                );
            }
        }

        /**
         * Render the page HTML
         */
        public function lsi_import_page_html() {
            ?>
            <div class="wrap">
                <h1 id="lightspeed-import-page"><?php echo WCLSI_IMPORTER_PAGE_TITLE ?></h1>
                <?php settings_errors( 'wclsi_settings' ); ?>
                <div id="wclsi-load-progress" class="updated">
					<span class="spinner wclsi-spinner"
                          style="float: left; width: inherit; margin: 5px 0 0 0;">&nbsp;</span>
                    <p>
                        <span id="wclsi-progress-msg"><?php echo __( 'Loading products ... ', 'woocommerce-lightspeed-pos' ) ?></span>
                        <b><span id="wclsi-progress-count">0%</span><?php echo __( ' complete', 'woocommerce-lightspeed-pos' ) ?>
                        </b>
                    </p>
                </div>
                <p>
                    <?php echo __( 'Import and Link Lightspeed Products into your WooCommerce store here!', 'woocommerce-lightspeed-pos' ); ?>
                    |
                    <a href="<?php echo WCLSI_ADMIN_SETTINGS_URL ?>"><?php echo __( 'Settings', 'woocommerce-lightspeed-pos' ) ?></a>
                    |
                    <a href="https://docs.woothemes.com/document/woocommerce-lightspeed-pos/" target="_blank">Documentation</a>
                    |
                    <a href="https://dzv365zjfbd8v.cloudfront.net/changelogs/woocommerce-lightspeed-pos/changelog.txt">v<?php echo WCLSI_VERSION ?></a>
                </p>
                <?php
                $last_sync = get_option( WCLSI_LAST_SYNC_TIMESTAMP );
                if ( false !== $last_sync ) {
                    $last_load_msg = __( 'Last automated sync run', 'woocommerce-lightspeed-pos' );

                    $timezone       = $this->ls_settings->store_timezone;
                    $last_load_time = new DateTime( 'now', new DateTimeZone( $timezone ) );
                    $last_load_time->setTimestamp( strtotime( $last_sync ) );
                    $last_load_formatted = $last_load_time->format( 'F j, H:i' );

                    $next_sync_time      = strtotime( $last_sync ) + ( 60 * 5 );
                    $next_sync_formatted = new DateTime( 'now', new DateTimeZone( $timezone ) );
                    $next_sync_formatted->setTimestamp( $next_sync_time );
                    $next_sync_formatted = $next_sync_formatted->format( 'F j, H:i' );

                    echo "<p><i>$last_load_msg: <b>$last_load_formatted - $timezone</b> - " .
                         "next run (approximately): <b>$next_sync_formatted - $timezone</b></i></p>";
                }

                if ( $this->wclsi_import_table->has_items() ) {
                    $load_button_txt = __( 'Re-Load products from Lightspeed', 'woocommerce-lightspeed-pos' );
                } else {
                    $load_button_txt = __( 'Load Products from Lightspeed', 'woocommerce-lightspeed-pos' );
                }
                $reload = $this->wclsi_import_table->has_items() ? 'data-reload="true"' : '';
                ?>
                <p>
                    <input
                            alt="#TB_inline?inlineId=wclsi_api_search&height=800&width=1200"
                            title="Search Lightspeed"
                            class="thickbox button button-primary help_tip tips"
                            data-tip="<?php echo __( 'Search your Lightspeed Store for specific products.' ) ?>"
                            type="button"
                            value="Search Lightspeed"
                        <?php echo $this->table_disabled ? 'disabled' : '' ?>
                    /> Or <input
                            id="wc-ls-load-prods"
                            type="button"
                            class="button button-secondary help_tip tips"
                            data-tip="<?php echo __( 'Loads Lightspeed products into a table list. No products are added to your WooCommerce store.' ) ?>"
                        <?php echo $reload ?>
                        <?php echo $this->table_disabled ? 'disabled' : '' ?>
                            value="<?php echo $load_button_txt; ?>"
                    />
                </p>
                <div id="wclsi_api_search" style="display: none;"><?php WCLSI_Api_Navigator::render_navigator(); ?>>
                </div>
                <form method="post" id="wc-ls-import-form" action="">
                    <input type="hidden" name="page" value="<?php echo htmlspecialchars( $_REQUEST['page'] ) ?>"/>
                    <?php wp_nonce_field( 'wclsi_ajax', 'wclsi_nonce', false ); ?>
                </form>
                <div id="wc-ls-import-results">
                    <?php $this->render_imported_prods() ?>
                </div>
            </div>
            <?php
        }

        /**
         * Render the imported products from LightSpeed and/or via the cached products
         */
        function render_imported_prods() {
            if ( ! $this->table_disabled ) {
                ?>
                <!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
                <form id="wc-imported-prods-filter" method="get">
                    <!-- For plugins, we also need to ensure that the form posts back to our current page -->
                    <input type="hidden" name="page" value="<?php echo htmlspecialchars( $_REQUEST['page'] ) ?>"/>
                    <?php

                    if ( $this->wclsi_import_table->has_items() ) {
                        $this->wclsi_import_table->search_box( 'Search name, price, sku, or category', 'wc_ls_search' );
                        $this->wclsi_import_table->display();
                    }
                    ?>
                </form>
                <?php
            }
        }
    }

    $LSI_Import_Page = new LSI_Import_page();
endif;
