<?php
if ( ! class_exists( 'WCLSI_Item_Image' ) ) :

	class WCLSI_Item_Image {

		function __construct() {
			add_action( 'delete_attachment', array( $this, 'sync_deleted_images' ) );
		}

		private static function get_item_image_mysql_args( $item_image, $wclsi_item_id ) {

			$args = array(
				'wclsi_item_id'  => $wclsi_item_id,
				'item_id'        => isset( $item_image->itemID ) ? $item_image->itemID : null,
				'item_matrix_id' => isset( $item_image->itemMatrixID ) ? $item_image->itemMatrixID : null,
				'image_id'       => isset( $item_image->imageID ) ? $item_image->imageID : null,
				'description'    => isset( $item_image->description ) ? $item_image->description : null,
				'filename'       => isset( $item_image->filename ) ? $item_image->filename : null,
				'ordering'       => isset( $item_image->ordering ) ? $item_image->ordering : null,
				'public_id'      => isset( $item_image->publicID ) ? $item_image->publicID : null,
				'base_image_url' => isset( $item_image->baseImageURL ) ? $item_image->baseImageURL : null,
				'size'           => isset( $item_image->size ) ? $item_image->size : null,
				'create_time'    => isset( $item_image->createTime ) ? $item_image->createTime : null,
				'time_stamp'     => isset( $item_image->timeStamp ) ? $item_image->timeStamp : null,
				'created_at'     => current_time( 'mysql' )
			);

			wclsi_format_empty_vals( $args );

			return $args;
		}

		static function init() {
			add_action( 'wclsi_import_images', array( get_called_class(), 'wclsi_import_images' ), 10, 3 );
		}

		/**
		 * @param Int $wc_prod_id
		 * @param array $ls_prod_images
		 *
		 * @return void
		 */
		public static function enqueue_item_images_to_download( int $wc_prod_id, array $ls_prod_images ) {
			// Spread the time for when the images get downloaded
			$time_spread = 1;


			usort( $ls_prod_images, function ( $a, $b ) {
				if ( $a->ordering > $b->ordering ) {
					return 1;
				} else {
					return - 1;
				}
			} );

			if ( count( $ls_prod_images ) > 0 && ! is_null( $ls_prod_images[0]->ordering ) && $ls_prod_images[0]->ordering != 0 ) {
				$ls_prod_images[0]->ordering = 0;
			}

			foreach ( $ls_prod_images as $ls_image ) {
				// only schedule an image to be imported if it is not already associated with the product
				if ( ! self::is_image_associated_with_wc_prod( $wc_prod_id, $ls_image ) ) {
					as_schedule_single_action( ( time() + $time_spread ++ ), 'wclsi_import_images', array(
						"wc_prod_id"        => $wc_prod_id,
						"ls_image_id"       => (int) $ls_image->image_id,
						"ls_image_ordering" => $ls_image->ordering
					), 'wclsi' );
				}
			}
		}

		private static function is_image_associated_with_wc_prod( int $wc_prod_id, $ls_image ) {
			$wc_prod = wc_get_product( $wc_prod_id );
			//if attachment id isn't set or the product doesn't exist, image is not currently associated with product
			if ( empty( $ls_image->wp_attachment_id ) || $wc_prod === false ) {
				return false;
			} else {
				//check if the image is the thumbnail, if yes then the image is already associated with the product
				$thumbnail_id = get_post_thumbnail_id( $wc_prod_id );
				if ( $ls_image->wp_attachment_id === $thumbnail_id and $ls_image->ordering == 0 ) {
					return true;
				}

				//check if image is in gallery, if yes then the image is already associated with the product
				$gallery_lookup_key          = $wc_prod->is_type( 'simple' ) || $wc_prod->is_type( 'variable' ) ? '_product_image_gallery' : 'woo_variation_gallery_images';
				$existing_gallery_image_data = get_post_meta( $wc_prod_id, $gallery_lookup_key, true );
				if ( is_array( $existing_gallery_image_data ) ) {
					$existing_gallery_image_ids = $existing_gallery_image_data;
				} else {
					$existing_gallery_image_ids = empty( $existing_gallery_image_data ) ? [] : explode( ',', $existing_gallery_image_data );
				}
				if ( array_search( $ls_image->wp_attachment_id, $existing_gallery_image_ids ) !== false ) {
					//if featured image has been changed, reprocess it
					if ( $ls_image->ordering == 0 ) {
						return false;
					}

					return true;
				}

				//if the image is not the thumbnail or in the gallery, the image is not already associated with the prod
				return false;

			}
		}

		/**
		 * Downloads the images associated with a woocommerce product from Lightspeed
		 *
		 * @param Int $wc_prod_id
		 * @param Int $ls_image_id - the "image_id" column in wclsi_item_images
		 *
		 * @return void
		 * @throws Exception
		 */
		public static function wclsi_import_images( int $wc_prod_id, int $ls_image_id, int $ls_image_ordering ) {
			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;
			$ls_image_row           = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $WCLSI_ITEM_IMAGES_TABLE WHERE image_id = $ls_image_id" ) );
			$ls_image_row->ordering = $ls_image_ordering;

			if ( empty( $ls_image_row ) ) {
				$msg = "Could not import an image for WooCommerce product $wc_prod_id, data not available for Lightspeed image $ls_image_id.";
				WCLSI_Logger::log_error( $msg );
				throw new Exception( $msg );
			}

			if ( ! empty( wp_attachment_is_image( $ls_image_row->wp_attachment_id ) ) ) {
				$attachment_id = $ls_image_row->wp_attachment_id;
			} else {
				$attachment_id = self::create_new_wc_image_attachment( $ls_image_row, $wc_prod_id );
			}

			if ( is_wp_error( $attachment_id ) ) {
				$msg = "Error importing image for WooCommerce product $wc_prod_id. Could not create new WP Attachment";
				WCLSI_Logger::log_error( $msg, $attachment_id->get_error_message() );

				return;
			}

			self::associated_ls_images_with_wc_prod( $attachment_id, $wc_prod_id, $ls_image_row->ordering );
		}

		/**
		 * Downloads a lightspeed image and generates a WC Image Attachment object
		 * Also update wclsi_item_images table with the resulting wp_attachment_id
		 *
		 * @param $ls_image_row
		 * @param $wc_prod_id
		 *
		 * @return int|WP_Error
		 */
		private static function create_new_wc_image_attachment( $ls_image_row, $wc_prod_id ) {

			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;

			$existing_image_attach_id = self::check_for_existing_image( $ls_image_row );
			if ( $existing_image_attach_id > 0 ) {
				$wpdb->update(
					$WCLSI_ITEM_IMAGES_TABLE,
					array( 'wp_attachment_id' => $existing_image_attach_id ),
					array( 'image_id' => $ls_image_row->image_id ),
					array( '%d' ),
					array( '%d' )
				);

				return $existing_image_attach_id;
			}

			if (
				isset( $ls_image_row->filename ) &&
				isset( $ls_image_row->base_image_url ) &&
				isset( $ls_image_row->public_id )
			) {

				$img_ext = pathinfo( $ls_image_row->filename, PATHINFO_EXTENSION );
				$img_url = $ls_image_row->base_image_url . self::get_wclsi_compression_strategy() . '/' . $ls_image_row->public_id . '.' . $img_ext;
				$result  = WCLSI_Item_Image::download_ls_image( $img_url, $ls_image_row->filename );

				if ( false === $result || is_wp_error( $result ) || ! empty( $result['error'] ) ) {

					$error_msg = "There was an issue with downloading images from Lightspeed! ";
					if ( is_array( $result ) && ! empty( $result['error'] ) ) {
						$error_msg .= $result['error'];
					} elseif ( is_wp_error( $result ) ) {
						$error_msg .= $result->get_error_message();
					} elseif ( false === $result ) {
						$error_msg .= 'The header value could not be determined via wp_remote_retrieve_header().';
					}

					if ( is_admin() ) {
						add_settings_error(
							'wclsi_settings',
							'file_get_contents_failed',
							$error_msg
						);
					}

					return new WP_Error( 'file_get_contents_failed', $error_msg );
				}

				if ( file_exists( $result['file'] ) ) {
					$attachment_id = wc_ls_create_attachment( $result['file'], $wc_prod_id, $ls_image_row->description );

					if ( is_wp_error( $attachment_id ) ) {
						return $attachment_id;
					}
					// Save ordering of the newly created attachment based off of the lightspeed ordering
					add_post_meta( $attachment_id, 'wclsi_lightspeed_item_image_order', $ls_image_row->ordering );

					$wpdb->update(
						$WCLSI_ITEM_IMAGES_TABLE,
						array( 'wp_attachment_id' => $attachment_id ),
						array( 'image_id' => $ls_image_row->image_id ),
						array( '%d' ),
						array( '%d' )
					);

					add_post_meta( $attachment_id, '_wclsi_image_public_id', $ls_image_row->public_id );

					return $attachment_id;
				}
			}

			return 0;
		}

		private static function check_for_existing_image( $prod_img ) {
			global $wpdb, $WP_POSTMETA;

			$existing_image_posts = $wpdb->get_row( "SELECT MAX( post_id ) FROM $WP_POSTMETA WHERE meta_key = '_wp_attached_file' AND meta_value LIKE '%$prod_img->public_id%'", ARRAY_N );
			if ( ! empty( $existing_image_posts ) ) {
				$wp_attach_id = $existing_image_posts[0];
			} else {
				$wp_attach_id = null;
			}

			if ( ! is_null( $wp_attach_id ) && ! is_wp_error( $wp_attach_id ) ) {
				return $wp_attach_id;
			} elseif ( is_wp_error( $wp_attach_id ) ) {
				WCLSI_Logger::log_error( 'error searching for attachment id for image', $wp_attach_id->get_error_message() );

				return;
			} else {
				return;
			}
		}

		/**
		 * Associates images that were downloaded from Lightspeed with their corresponding imported WC Prod
		 *
		 * @param $wc_prod_id
		 * @param $ls_image_row - the row in the wclsi_item_images table
		 * @param $ordering - ordering of the image
		 *
		 * @return void
		 */
		private static function associated_ls_images_with_wc_prod( $attachment_id, $wc_prod_id, $ordering ) {
			if ( ! empty( $attachment_id ) ) {
				$wc_prod = wc_get_product( $wc_prod_id );

				if ( empty( $wc_prod ) ) {
					WCLSI_Logger::log_error( "Error importing image. Could not find associated product $wc_prod_id" );

					return;
				}

				$gallery_lookup_key          = $wc_prod->is_type( 'simple' ) || $wc_prod->is_type( 'variable' ) ? '_product_image_gallery' : 'woo_variation_gallery_images';
				$existing_gallery_image_data = get_post_meta( $wc_prod_id, $gallery_lookup_key, true );
				if ( is_array( $existing_gallery_image_data ) ) {
					$existing_gallery_image_ids = $existing_gallery_image_data;
				} else {
					$existing_gallery_image_ids = empty( $existing_gallery_image_data ) ? [] : explode( ',', $existing_gallery_image_data );
				}

				//check for featured image in the gallery, if it exists delete it
				$key = array_search( $attachment_id, $existing_gallery_image_ids );
				if ( $key !== false ) {
					unset( $existing_gallery_image_ids[ $key ] );
				}

				if ( $ordering === 0 || $ordering === "0" ) {
					set_post_thumbnail( $wc_prod_id, $attachment_id );
					update_post_meta( $wc_prod_id, $gallery_lookup_key, implode( ',', $existing_gallery_image_ids ) );

					return;
				}


				if ( $wc_prod->is_type( 'simple' ) ) {

					// TODO: sort images based on ordering from wclsi_item_images - walk down array until there it's empty of it's the last one
					// technically ordering should be unique, so this *should* work, but we should check if there is something in the index
					array_splice( $existing_gallery_image_ids, $ordering, 0, $attachment_id );
					update_post_meta( $wc_prod_id, $gallery_lookup_key, implode( ',', $existing_gallery_image_ids ) );
				}


				if ( $wc_prod->is_type( 'variable' ) ) {

					$existing_gallery_image_ids [] = intval( $attachment_id );
					$unique_gallery_image_array    = array_unique( $existing_gallery_image_ids );
					update_post_meta( $wc_prod_id, $gallery_lookup_key, implode( ',', $unique_gallery_image_array ) );
				}


				$wc_prod            = wc_get_product( $wc_prod_id );
				$has_parent_product = wc_get_product( $wc_prod->get_parent_id() );
				if ( $has_parent_product ) {
					$gallery_image_array          = [];
					$existing_gallery_image_array = get_post_meta( $wc_prod_id, $gallery_lookup_key, true );

					foreach ( $existing_gallery_image_array as $existing_image ) {
						$gallery_image_array[] = $existing_image;
					}

					$gallery_image_array[] = intval( $attachment_id );
					update_post_meta( $wc_prod_id, $gallery_lookup_key, $gallery_image_array );
				}
			}
		}

		private static function get_wclsi_compression_strategy() {
			global $WCLSI;

			if ( key_exists( 'wclsi_image_compression_strategy', $WCLSI->settings ) ) {
				return $WCLSI->settings['wclsi_image_compression_strategy'];
			}

			return 'q_auto:eco';
		}

		public static function insert_item_images( $item ) {
			if ( ! isset( $item->Images ) ) {
				return;
			}

			if ( is_array( $item->Images->Image ) ) {
				foreach ( $item->Images->Image as $image ) {
					self::insert_item_image( $image, $item->wclsi_item_id );
				}
			} else if ( is_object( $item->Images->Image ) ) {
				self::insert_item_image( $item->Images->Image, $item->wclsi_item_id );
			}
		}

		public static function update_item_images( $new_item, $old_item ) {

			if ( empty( $new_item->Images->Image ) ) {
				return;
			}

			if ( is_object( $new_item->Images->Image ) ) {
				$array_format            = array( $new_item->Images->Image );
				$new_item->Images->Image = $array_format;
			}

			if ( is_array( $new_item->Images->Image ) ) {
				foreach ( $new_item->Images->Image as $image ) {
					if ( self::item_image_exists( $image->imageID, $old_item ) ) {
						// @todo not sure if this is necessary since not sure what kind of image updates can be performed in LS
						self::update_item_image( $image, $old_item->id );
					} else {
						self::insert_item_image( $image, $old_item->id );
					}
				}
			}

			self::prune_old_item_images( $new_item, $old_item );
		}

		public static function insert_item_image( $item_image, $wclsi_item_id ) {
			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;

			if ( is_null( $item_image ) ) {
				return;
			}

			$args = self::get_item_image_mysql_args( $item_image, $wclsi_item_id );

			$formatting = array(
				'%d',
				'%d',
				'%d',
				'%d',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
				'%s'
			);

			$wpdb->insert( $WCLSI_ITEM_IMAGES_TABLE, $args, $formatting );
		}

		public static function update_item_image( $new_item_image, $wclsi_item_id ) {
			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;

			$update_args            = self::get_item_image_mysql_args( $new_item_image, $wclsi_item_id );
			$where_args['image_id'] = $new_item_image->imageID;

			if ( isset( $update_args['created_at'] ) ) {
				unset( $update_args['created_at'] );
			}

			$where_format  = array( '%d' );
			$update_format = array(
				'%d',
				'%d',
				'%d',
				'%d',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
				'%s'
			);

			$wpdb->update( $WCLSI_ITEM_IMAGES_TABLE, $update_args, $where_args, $update_format, $where_format );
		}

		private static function item_image_exists( $image_id, WCLSI_Lightspeed_Prod $item ) {
			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;
			$orphaned_image_id = $wpdb->get_var( "SELECT image_id FROM $WCLSI_ITEM_IMAGES_TABLE WHERE image_id = $image_id" );

			foreach ( $item->images as $image ) {
				if ( $image->image_id == $image_id ) {
					return true;
				}
			}

			if ( $orphaned_image_id == $image_id ) {
				return true;
			}

			return false;
		}

		private static function missing_from_new_item_images( $old_image_id, $new_item ) {
			foreach ( $new_item->Images->Image as $new_item_image ) {
				if ( $new_item_image->imageID == $old_image_id ) {
					return false;
				}
			}

			return true;
		}

		/**
		 * During a sync operation, check if we need to delete any item images.
		 *
		 * @param $new_item
		 * @param $old_item
		 */
		private static function prune_old_item_images( $new_item, $old_item ) {
			foreach ( $old_item->images as $old_image ) {
				if ( self::missing_from_new_item_images( $old_image->image_id, $new_item ) ) {
					self::delete_item_image( $old_image );
				}
			}
		}

		public static function delete_item_image( $old_image ) {
			global $wpdb, $WCLSI_ITEM_IMAGES_TABLE;

			if ( $old_image->wp_attachment_id > 0 ) {
				wp_delete_attachment( $old_image->wp_attachment_id, true );
			}

			$wpdb->delete( $WCLSI_ITEM_IMAGES_TABLE, array( 'image_id' => $old_image->image_id ), array( '%d' ) );
		}

		public static function download_ls_image( $image_url, $filename ) {

			$image = $image_url;

			$get = wp_remote_get( $image );

			if ( is_wp_error( $get ) ) {
				return $get;
			}

			$type = wp_remote_retrieve_header( $get, 'content-type' );

			if ( is_wp_error( $type ) ) {
				return $get;
			}

			if ( ! $type ) {
				return false;
			}

			return wp_upload_bits( basename( $filename ), '', wp_remote_retrieve_body( $get ) );
		}
	}

	WCLSI_Item_Image::init();
endif;
