<?php
if ( ! class_exists( 'WC_LS_Upgrade_Routines' ) ):

    class WC_LS_Upgrade_Routines {

        public static function perform_upgrades() {
            global $wpdb;
            $wclsi_version = $wpdb->get_var( "SELECT option_value FROM {$wpdb->options} WHERE option_name='wclsi_version'" ) ?? WCLSI_VERSION;
            if ( WCLSI_VERSION != $wclsi_version ) {
                self::run_1_6_1_routine( $wclsi_version );
                self::run_1_7_7_routine( $wclsi_version );
                self::run_1_8_1_routine( $wclsi_version );
                self::run_1_8_2_routine( $wclsi_version );
                self::run_1_8_3_routine( $wclsi_version );
                self::run_1_9_0_routine( $wclsi_version );
                self::run_1_9_1_routine( $wclsi_version );
                self::run_1_9_2_routine( $wclsi_version );
                self::run_1_9_6_routine( $wclsi_version );
                self::run_2_0_0_routine( $wclsi_version );
                self::run_2_0_1_routine( $wclsi_version );
                self::run_2_1_0_routine( $wclsi_version );
                self::run_2_2_0_routine( $wclsi_version );
                self::run_2_7_0_routine( $wclsi_version );
                self::run_2_8_0_routine( $wclsi_version );
                self::run_2_11_0_routine( $wclsi_version );

                update_option( 'wclsi_version', WCLSI_VERSION, false );
            }
        }

        public static function run_2_11_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.11.0' ) >= 0 ) {
                return;
            }
            global $WCLSI;

            if ( ! array_key_exists( 'wclsi_handle_archived_products', $WCLSI->settings ) ) {
                $WCLSI->settings['wclsi_handle_archived_products'] = 'do nothing';
                update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
            }

        }

        public static function run_2_8_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.8.0' ) >= 0 ) {
                return;
            }

            global $WCLSI;

            $WCLSI->settings['wclsi_debug_mode'] = false;
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_2_7_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.7.0' ) >= 0 ) {
                return;
            }

            global $WCLSI;

            // transfer obsolete setting status to new remove default category setting and
            // delete old setting
            $remove_uncategorized_category_setting            = $WCLSI->settings['wclsi_remove_uncategorized_category'];
            $WCLSI->settings['wclsi_remove_default_category'] = $remove_uncategorized_category_setting;
            unset( $WCLSI->settings['wclsi_remove_uncategorized_category'] );
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_2_2_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.2.0' ) >= 0 ) {
                return;
            }


            global $WCLSI;

            // set sales syncing to false if not opted in
            $sales_sync_setting = get_option( WCLSI_SALES_SYNC );
            if ( $sales_sync_setting == false ) {
                $WCLSI->settings[ WCLSI_SALES_SYNC ] = false;
                update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
            }

            $WCLSI->settings[ WCLSI_REPORT_ERRORS ] = true;
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );

        }

        public static function run_2_1_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.1.0' ) >= 0 ) {
                return;
            }

            /**
             * Add Beta Sales Syncing - customer must opt in.
             */
            global $WCLSI;
            $WCLSI->settings[ WCLSI_SALES_SYNC ] = 'false';
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_2_0_1_routine( $current_version ) {
            if ( version_compare( $current_version, '2.0.1' ) >= 0 ) {
                return;
            }

            global $WCLSI;
            $WCLSI->settings[ WCLSI_WC_SELECTIVE_SYNC ]['tags'] = 'true';
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_2_0_0_routine( $current_version ) {
            if ( version_compare( $current_version, '2.0.0' ) >= 0 ) {
                return;
            }

            global $WCLSI;
            if ( isset( $WCLSI->settings[ WCLSI_POLLER_SETTING ] ) ) {
                $WCLSI->settings[ WCLSI_POLLER_SETTING ]               = array();
                $WCLSI->settings[ WCLSI_POLLER_SETTING ]['is_enabled'] = 'true';
                $WCLSI->settings[ WCLSI_POLLER_SETTING ]['interval']   = 5;
            }

            if ( isset( $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ] ) ) {
                $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ]               = array();
                $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ]['is_enabled'] = 'true';
                $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ]['interval']   = 30;
            }

            if ( isset( $WCLSI->settings[ WCLSI_IMPORT_ON_AUTOLOAD ] ) ) {
                switch ( $WCLSI->settings[ WCLSI_IMPORT_ON_AUTOLOAD ] ) {
                    case 'do_nothing':
                        $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ][ WCLSI_AUTOLOAD_AFTER_ACTION ] = WCLSI_Check_For_New_Prods::LOAD_INTO_IMPORTER;
                        break;
                    case 'import_and_sync':
                        $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ][ WCLSI_AUTOLOAD_AFTER_ACTION ] = WCLSI_Check_For_New_Prods::IMPORT_AND_LINK;
                        break;
                    case 'import':
                        $WCLSI->settings[ WCLSI_LS_TO_WC_AUTOLOAD ][ WCLSI_AUTOLOAD_AFTER_ACTION ] = WCLSI_Check_For_New_Prods::IMPORT;
                        break;
                }
                unset( $WCLSI->settings[ WCLSI_IMPORT_ON_AUTOLOAD ] );
            }

            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_1_9_6_routine( $current_version ) {
            if ( version_compare( $current_version, '1.9.6' ) >= 0 ) {
                return;
            }

            /**
             * There might be some sites out there that did not create the attribute_set
             * table. Let's re-run these migrations in case this table wasn't created properly.
             *
             * Note: all the below methods have table_exists() and col_exists() checks before running
             */
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            $wclsi_tables          = new Create_WCLSI_Tables();
            $wclsi_attrs_migration = new Add_Wc_Attr_Id_Cols();
            $wclsi_tables->create_wclsi_item_attribute_sets();
            $wclsi_attrs_migration->add_attr_wc_id_columns();
        }

        public static function run_1_9_2_routine( $current_version ) {
            if ( version_compare( $current_version, '1.9.2' ) >= 0 ) {
                return;
            }

            /**
             * Uncheck LS "Sale Price" if the customer has the setting enabled, but has not set it up in Lightspeed
             */
            global $WCLSI;
            if ( empty( $WCLSI->ls_account_id ) ) {
                // If the store has not been set up yet, don't do anything.
                return;
            }

            $selective_sync = $WCLSI->settings[ WCLSI_LS_SELECTIVE_SYNC ];
            if ( ! $WCLSI->sale_price_level_enabled() && ( 'true' === $selective_sync['sale_price'] ) ) {
                unset( $WCLSI->settings[ WCLSI_LS_SELECTIVE_SYNC ]['sale_price'] );
                update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
            }
        }

        public static function run_1_9_1_routine( $current_version ) {
            if ( version_compare( $current_version, '1.9.1' ) >= 0 ) {
                return;
            }

            global $WCLSI;
            $selective_sync = $WCLSI->settings[ WCLSI_LS_SELECTIVE_SYNC ];
            if ( ! is_null( $selective_sync ) && array_key_exists( 'stock_quantity_checkout', $selective_sync ) ) {
                if ( 'true' === $selective_sync['stock_quantity_checkout'] && ! key_exists( 'stock_quantity', $selective_sync ) ) {
                    $WCLSI->settings[ WCLSI_LS_SELECTIVE_SYNC ]['stock_quantity'] = 'true';
                    unset( $WCLSI->settings[ WCLSI_LS_SELECTIVE_SYNC ]['stock_quantity_checkout'] );
                    update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
                }
            }
        }

        public static function run_1_9_0_routine( $current_version ) {
            if ( version_compare( $current_version, '1.9.0' ) >= 0 ) {
                return;
            }

            /**
             * In version 1.8.1 - we used the wrong option name, let's run this again with the right option name
             */
            delete_option( 'wclsi_update_cancelled_order' );
        }

        public static function run_1_8_3_routine( $current_version ) {
            if ( version_compare( $current_version, '1.8.3' ) >= 0 ) {
                return;
            }

            /**
             * Clear out bad item_e_commerce (webstore) data that does not have an item_e_commerce_id
             * This will effectively "reset" the webstore data for products and allow them to properly
             * be inserted into the DB
             */
            global $wpdb, $WCLSI_ITEM_E_COMMERCE_TABLE;
            $wpdb->query( "DELETE FROM $WCLSI_ITEM_E_COMMERCE_TABLE WHERE item_e_commerce_id IS NULL" );
        }

        public static function run_1_8_2_routine( $current_version ) {
            if ( version_compare( $current_version, '1.8.2' ) >= 0 ) {
                return;
            }

            update_option( WCLSI_LAST_LOAD_TIMESTAMP, date( DATE_ATOM, strtotime( 'now' ) ) );
        }

        public static function run_1_8_1_routine( $current_version ) {
            if ( version_compare( $current_version, '1.8.1' ) >= 0 ) {
                return;
            }

            /**
             * In version 1.7.9 - this option was misplaced and could result in a complete
             * halt in calls to update_ls_prod() - let's delete this to be safe
             */
            delete_option( 'wclsi_order_cancelled' );
        }

        public static function run_1_7_7_routine( $current_version ) {
            if ( version_compare( $current_version, '1.7.7' ) >= 0 ) {
                return;
            }

            global $WCLSI;
            $WCLSI->settings[ WCLSI_WC_SELECTIVE_SYNC ]['attributes'] = 'true';
            update_option( $WCLSI->get_settings_option_key(), $WCLSI->settings );
        }

        public static function run_1_6_1_routine( $current_version ) {
            if ( version_compare( $current_version, '1.6.1' ) >= 0 ) {
                return;
            }

            // Reset the lock so it follows the 30s max rule
            delete_transient( 'wclsi_api_call_lock' );
        }
    }

endif;
