<?php

if ( ! class_exists( 'WCLSI_Woo_Prod_Page' ) ) :

    class WCLSI_Woo_Prod_Page {
        function __construct() {
            add_action( 'admin_notices', array( $this, 'show_wclsi_prod_notices' ), 10 );
        }

        function show_wclsi_prod_notices() {
            $current_screen = get_current_screen();
            if ( 'product' !== $current_screen->id ) {
                return;
            }

            $wc_prod = wc_get_product( get_the_ID() );
            if ( ! $wc_prod ) {
                return;
            }

            $this->show_active_sync_notice( $wc_prod );
            $this->show_image_import_in_progress_notice( $wc_prod );
            $this->show_ls_archived_notice( $wc_prod );
            $this->show_action_scheduler_error( $wc_prod );
        }

        private function show_image_import_in_progress_notice( WC_Product $wc_prod ) {
            $wc_prod_id = $wc_prod->get_id();

            $is_importing = WCLSI_Lightspeed_Prod::is_importing_images_for_product( $wc_prod_id );

            if ( $is_importing ) {
                $warning        = __( 'Images for this product are importing from Lightspeed', 'woocommerce-lightspeed-pos' );
                $recommendation = __( 'They will be available as soon as the download completes', 'woocommerce-lightspeed-pos' );
                ?>
                <div class="notice is-dismissible notice-warning">
                    <p>
                        <strong><?php echo $warning ?></strong>.
                        <?php echo $recommendation ?>
                    </p>
                </div>
                <?php
            }
        }


        private function show_active_sync_notice( WC_Product $wc_prod ) {
            if ( WCLSI_Lightspeed_Prod::is_imported( $wc_prod->get_id() ) ) {
                $img_src        = plugins_url( "assets/img/ls-icon50x50.png", dirname( __FILE__ ) );
                $warning        = __( 'Warning: this product is actively linked with Lightspeed', 'woocommerce-lightspeed-pos' );
                $recommendation = __( 'It is recommended to go into draft mode or hide the product first before making any changes to it!', 'woocommerce-lightspeed-pos' );
                ?>
                <div class="notice is-dismissible notice-warning">
                    <p>
                        <img src="<?php echo $img_src ?>" alt="<?php echo $warning ?>"
                             style="height: 20px; width: 20px; vertical-align: bottom;"/>
                        <strong><?php echo $warning ?></strong>.
                        <?php echo $recommendation ?>
                    </p>
                </div>
                <?php
            }
        }

        private function show_action_scheduler_error( WC_Product $wc_prod ) {
            $action_scheduler_error = get_transient( "wclsi_action_scheduler_failed_for_{$wc_prod->get_id()}" );

            if ( $action_scheduler_error != false ) {
                $img_src        = plugins_url( "assets/img/ls-icon50x50.png", dirname( __FILE__ ) );
                $warning        = __( 'This product has encountered the following action scheduler error when attempting
					to sync to Lightspeed', 'woocommerce-lightspeed-pos' );
                $recommendation = __( $action_scheduler_error, 'woocommerce-lightspeed-pos' );
                ?>
                <div class="notice is-dismissible notice-warning">
                    <p>
                        <img src="<?php echo $img_src ?>" alt="<?php echo $warning ?>"
                             style="height: 20px; width: 20px; vertical-align: bottom;"/>
                        <strong><?php echo $warning ?></strong>.
                        <?php echo $recommendation ?>
                    </p>
                </div>
                <?php
            }
        }

        /**
         * Renders
         *
         * @param WC_Product $wc_prod
         */
        private function show_ls_archived_notice( WC_Product $wc_prod ) {
            global $WCLSI;
            $archived_handler_setting = $WCLSI->settings['wclsi_handle_archived_products'];

            $wclsi_item = new WCLSI_Lightspeed_Prod();
            $wclsi_item->init_via_wc_prod_id( get_the_ID() );
            if ( $wclsi_item->id > 0 && $wclsi_item->archived && $archived_handler_setting != 'delete' ) {
                ?>
                <div class="notice error is-dismissible">
                    <p>Warning: this is an <strong>Archived Lightspeed Product!</strong></p>
                    <p>It is recommended to <strong>hide this product from your store front</strong> if you have not
                        already done so!</p>
                </div>
                <?php
            }

            if ( $wclsi_item->id > 0 && $wclsi_item->archived && $archived_handler_setting == 'delete' ) {
                ?>
                <div class="notice error is-dismissible">
                    <p>Warning: this is an <strong>Archived Lightspeed Product!</strong></p>
                    <p>The archived product handler will not process a product removal from the edit product page.</p>
                    <p>Please run a product update on the <strong><a href='admin.php?page=wclsi'>Lightspeed Importer
                                Page</a></strong> to process the product's removal.</p>
                </div>
                <?php
            }

            if ( $wclsi_item->is_matrix_product() ) {
                $wc_prod       = wc_get_product( get_the_ID() );
                $variation_ids = $wc_prod->get_children();
                if ( empty( $variation_ids ) ) {
                    return;
                }

                $archived_children = [];
                foreach ( $variation_ids as $variation_id ) {
                    $child_ls_prod = new WCLSI_Lightspeed_Prod();
                    $child_ls_prod->init_via_wc_prod_id( $variation_id );
                    if ( $child_ls_prod->id > 0 && $child_ls_prod->wc_prod_id > 0 && $child_ls_prod->archived ) {
                        $wc_variation = wc_get_product( $child_ls_prod->wc_prod_id );
                        if ( $wc_variation->get_id() > 0 ) {
                            $archived_children[] = $wc_variation->get_formatted_name();
                        }
                    }
                }

                if ( ! empty( $archived_children ) ) {
                    ?>
                    <div class="error is-dismissible'">
                        <p>Warning: there are one or more variations that are <strong>archived lightspeed
                                products!</strong></p>
                        <p>The following variations have been archived in Lightspeed:</p>
                        <ul style="list-style: circle; margin-left: 30px;">
                            <?php foreach ( $archived_children as $archived_child ) {
                                echo "<li>{$archived_child}</li>";
                            } ?>
                        </ul>
                        <p>It is recommended to <strong>hide the variations from your store front</strong> if you have
                            not already done so!</p>
                    </div>
                    <?php
                }
            }
        }
    }

    global $WCLSI_Woo_Prod_Page;
    $WCLSI_Woo_Prod_Page = new WCLSI_Woo_Prod_Page();
endif;
