<?php

if ( ! class_exists( 'WCLSI_Logger' ) ) :
    class WCLSI_Logger {
        const WCLSI_SYNC_EVENTS_LOG = 'wclsi-sync-events-log';
        const WCLSI_DEBUG_LOG = 'wclsi-debug-log';
        const WCLSI_ERRORS_LOG = 'wclsi-errors-log';

        static function log_error( string $msg, $data = null, $show_stack = false ) {
            self::log_event( $msg, self::WCLSI_ERRORS_LOG, $data, $show_stack );
        }

        static function log_sync_event( string $msg, $data = null ) {
            self::log_event( $msg, self::WCLSI_SYNC_EVENTS_LOG, $data, false );
        }

        static function log_debug_event( string $msg, $data = null, $show_stack = false ) {
            if ( wclsi_debug() ) {
                self::log_event( $msg, self::WCLSI_DEBUG_LOG, $data, $show_stack );
            }
        }

        private static function log_event( string $msg, $logger, $data = null, $show_stack = true ): void {
            global $WCLSI_WC_Logger;
            if ( empty( $WCLSI_WC_Logger ) ) {
                $WCLSI_WC_Logger = new WC_Logger();
            }

            $formatted_msg = $show_stack ?
                sprintf( "%s\n%s\n%s\n", $msg, print_r( $data, true ), self::get_stack_trace() ) :
                sprintf( "%s\n%s\n", $msg, print_r( $data, true ) );

            $WCLSI_WC_Logger->add( $logger, $formatted_msg );
        }

        private static function get_stack_trace() {
            ob_start();
            debug_print_backtrace( DEBUG_BACKTRACE_IGNORE_ARGS, 25 );

            return ob_get_clean();
        }
    }

endif;
